/*
   This client implements only GET.

   This client takes
      hostname  port  path
   as optional command-line arguments.
*/

import java.io.InputStreamReader;
import java.io.BufferedReader;
import java.io.PrintWriter;
import java.io.IOException;
import java.net.*;

public class HttpClient_v1
{
   private static final int SERVER_PORT = 8080; // Should be above 1023.

   public static void main (String[] args)
   {
      final String hostName;
      if (args.length > 0)
      {
         hostName = args[0];
      }
      else
      {
         hostName = "localhost";
      }

      final int portNumber;
      if (args.length > 1)
      {
         portNumber = Integer.parseInt(args[1]);
      }
      else
      {
         portNumber = SERVER_PORT;
      }

      final String path;
      if (args.length > 2)
      {
         path = args[2];
      }
      else
      {
         path = "/";
      }

      // Get this client's process id number (PID). This helps
      // to identify the client in the server's transcrip.
      final ProcessHandle handle = ProcessHandle.current();
      final long pid = handle.pid();
      System.out.println("CLIENT: Process ID number (PID): " + pid );

      // Make a connection to the server.
      Socket         socket = null;
      BufferedReader in = null;
      PrintWriter    out = null;
      try
      {
         System.out.println("CLIENT: connecting to server: " + hostName + " on port " + portNumber );
         socket = new Socket(InetAddress.getByName(hostName), portNumber);

         in = new BufferedReader(
                  new InputStreamReader(
                       socket.getInputStream()));
         out = new PrintWriter(socket.getOutputStream());
      }
      catch (IOException e)
      {
         System.out.println("CLIENT: Cannot connect to server.");
         e.printStackTrace();
         System.exit(-1);
      }

      // Send the server a request.
             out.println("GET " + path + " HTTP/1.1");
      System.out.println("<GET " + path + " HTTP/1.1");
             out.println("Host: " + hostName + ":" + portNumber);
      System.out.println("<Host: " + hostName + ":" + portNumber);
             out.println("Connection: close");
      System.out.println("<Connection: close");
             out.println();
      System.out.println("<");
             out.flush();

      // Receive the response from the server (including the response headers).
      try
      {
         String oneLine;
         // Read the response headers.
         while ( (oneLine = in.readLine()) != null )
         {
            System.out.println(">" + oneLine);
            if ( oneLine.isEmpty() )
            {
               break;
            }
         }
         // Read the entity body.
         while ( (oneLine = in.readLine()) != null )
         {
            System.out.println(oneLine);
         }
      }
      catch (IOException e)
      {
         System.out.println("CLIENT: Cannot receive response from server.");
         System.out.println( e );
      }

      // Disconnect from the server.
      try
      {
         socket.close();
         System.out.println("CLIENT: Closed socket.");
      }
      catch (IOException e)
      {
         System.out.println("CLIENT: Cannot disconnect from server.");
         System.out.println( e );
      }
   }
}
