/*
   This client uses Java's "high level" abstractions
   for communicating with an HTTP server, the URL,
   URLConnection, and HttpUrlConnection classes.
   See
      https://docs.oracle.com/en/java/javase/21/docs/api/java.base/java/net/URL.html
      https://docs.oracle.com/en/java/javase/21/docs/api/java.base/java/net/URLConnection.html
      https://docs.oracle.com/en/java/javase/21/docs/api/java.base/java/net/HttpURLConnection.html

   This client takes a URL as an optional command-line argument.
*/

import java.io.InputStreamReader;
import java.io.BufferedReader;
import java.io.IOException;
import java.net.URL;

public class HttpClient_v3
{
   public static void main (String[] args)
   {
      final String resourceURL;
      if (args.length > 0)
      {
         resourceURL = args[0];
      }
      else
      {
         resourceURL = "http://localhost:8080/";
      }

      // Get this client's process id number (PID). This helps
      // to identify the client in the server's transcrip.
      final ProcessHandle handle = ProcessHandle.current();
      final long pid = handle.pid();
      System.out.println("CLIENT: Process ID number (PID): " + pid );


      // Send a request to the server.
      System.out.println("CLIENT: retrieving resource: " + resourceURL );
      try
      {
         final URL url = new URL(resourceURL);
         final BufferedReader in = new BufferedReader(
                                     new InputStreamReader(
                                       url.openStream()));
         String line;
         while ( (line = in.readLine()) != null )
         {
            System.out.println(line);
         }
         in.close();
      }
      catch (IOException e)
      {
         System.out.println("CLIENT: Error occurred while retrieving URL.");
         System.out.println(e);
      }
   }
}
