/*
   This client uses Java's "high level" abstractions
   for communicating with an HTTP server, the URL,
   URLConnection, and HttpUrlConnection classes.
   See
      https://docs.oracle.com/en/java/javase/21/docs/api/java.base/java/net/URL.html
      https://docs.oracle.com/en/java/javase/21/docs/api/java.base/java/net/URLConnection.html
      https://docs.oracle.com/en/java/javase/21/docs/api/java.base/java/net/HttpURLConnection.html

   This client takes a resource from the html_public folder as an optional command-line argument.
*/

import java.io.InputStreamReader;
import java.io.BufferedReader;
import java.io.IOException;
import java.net.URL;
import java.net.URLConnection;
import java.net.HttpURLConnection;

public class HttpClient_v4
{
   public static void main (String[] args)
   {
      final String resourceURL;
      if (args.length > 0)
      {
         resourceURL = "http://localhost:8080/" + args[0];
      }
      else
      {
         resourceURL = "http://localhost:8080/static.html";
      }

      // Get this client's process id number (PID). This helps
      // to identify the client in the server's transcrip.
      final ProcessHandle handle = ProcessHandle.current();
      final long pid = handle.pid();
      System.out.println("CLIENT: Process ID number (PID): " + pid );


      // Send a few requests to the server.
      System.out.println("CLIENT: retrieving resource: " + resourceURL );
      try
      {
         final URL url = new URL(resourceURL);

         final URLConnection connection = url.openConnection();
         System.out.println("CLIENT: created connection: " + connection );

         final String mimeType = connection.getContentType();
         System.out.println("CLIENT: URL content type = " + mimeType);

         final int contentLength = connection.getContentLength();
         System.out.println("CLIENT: URL content length = " + contentLength);

         final Object content = connection.getContent();
         System.out.println("CLIENT: content is an instance of = " + content.getClass());
         System.out.println("CLIENT: URL content = " + content);

         //final HttpURLConnection httpConnection = (HttpURLConnection) connection;

         final BufferedReader in = new BufferedReader(
                                     new InputStreamReader(
                                       connection.getInputStream()));
         String line;
         while ( (line = in.readLine()) != null )
         {
            System.out.println(line);
         }
         in.close();
      }
      catch (IOException e)
      {
         System.out.println("CLIENT: Error occurred while retrieving URL.");
         System.out.println(e);
      }
   }
}
